#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/11/4 18:00
Desc: 东方财富网-概念板-行情中心-日K-筹码分布
https://quote.eastmoney.com/concept/sz000001.html
"""

from datetime import datetime

import pandas as pd
import py_mini_racer
import requests


def stock_cyq_em(symbol: str = "000001", adjust: str = "") -> pd.DataFrame:
    """
    东方财富网-概念板-行情中心-日K-筹码分布
    https://quote.eastmoney.com/concept/sz000001.html
    :param symbol: 股票代码
    :type symbol: str
    :param adjust: choice of {"qfq": "前复权", "hfq": "后复权", "": "不复权"}
    :type adjust: str
    :return: 筹码分布
    :rtype: pandas.DataFrame
    """
    html_str = """
    // @ts-nocheck

    /**
     * 计算分布及相关指标
     * @param {number} index 当前选中的K线的索引
     * @return {{x: Array.<number>, y: Array.<number>}}
     */
    /**
    this.range = 120;
    */
    function CYQCalculator(index, klinedata) {
        var maxprice = 0;
        var minprice = 0;
        var factor = 150;
        var start = this.range ? Math.max(0, index - this.range + 1) : 0;
        /**
         * K图数据[time,open,close,high,low,volume,amount,amplitude,turnoverRate]
         */
        var kdata = klinedata.slice(start, Math.max(1, index + 1));
        if (kdata.length === 0) throw 'invaild index';
        for (var i = 0; i < kdata.length; i++) {
            var elements = kdata[i];
            maxprice = !maxprice ? elements.high : Math.max(maxprice, elements.high);
            minprice = !minprice ? elements.low : Math.min(minprice, elements.low);
        }

        // 精度不小于0.01 产品逻辑
        var accuracy = Math.max(0.01, (maxprice - minprice) / (factor - 1));
        /**
         * 值域
         * @type {Array.<number>}
         */
        var yrange = [];
        for (var i = 0; i < factor; i++) {
            yrange.push((minprice + accuracy * i).toFixed(2) / 1);
        }
        /**
         * 横轴数据
         */
        var xdata = createNumberArray(factor);

        for (var i = 0; i < kdata.length; i++) {
            var eles = kdata[i];

            var open = eles.open,
                close = eles.close,
                high = eles.high,
                low = eles.low,
                avg = (open + close + high + low) / 4,
                turnoverRate = Math.min(1, eles.hsl / 100 || 0);

            var H = Math.floor((high - minprice) / accuracy),
                L = Math.ceil((low - minprice) / accuracy),
                // G点坐标, 一字板时, X为进度因子
                GPoint = [high == low ? factor - 1 : 2 / (high - low), Math.floor((avg - minprice) / accuracy)];
            // 衰减
            for (var n = 0; n < xdata.length; n++) {
                xdata[n] *= (1 - turnoverRate);
            }

            if (high == low) {
                // 一字板时，画矩形面积是三角形的2倍
                xdata[GPoint[1]] += GPoint[0] * turnoverRate / 2;
            } else {
                for (var j = L; j <= H; j++) {
                    var curprice = minprice + accuracy * j;
                    if (curprice <= avg) {
                        // 上半三角叠加分布分布
                        if (Math.abs(avg - low) < 1e-8) {
                            xdata[j] += GPoint[0] * turnoverRate;
                        } else {
                            xdata[j] += (curprice - low) / (avg - low) * GPoint[0] * turnoverRate;
                        }
                    } else {
                        // 下半三角叠加分布分布
                        if (Math.abs(high - avg) < 1e-8) {
                            xdata[j] += GPoint[0] * turnoverRate;
                        } else {
                            xdata[j] += (high - curprice) / (high - avg) * GPoint[0] * turnoverRate;
                        }
                    }
                }
            }

        }


        var currentprice = klinedata[index].close;
        var totalChips = 0;
        for (var i = 0; i < factor; i++) {
            var x = xdata[i].toPrecision(12) / 1;
            //if (x < 0) xdata[i] = 0;
            totalChips += x;
        }
        var result = new CYQData();
        result.x = xdata;
        result.y = yrange;
        result.benefitPart = result.getBenefitPart(currentprice);
        result.avgCost = getCostByChip(totalChips * 0.5).toFixed(2);
        result.percentChips = {
            '90': result.computePercentChips(0.9),
            '70': result.computePercentChips(0.7)
        };
        return result;

        /**
         * 获取指定筹码处的成本
         * @param {number} chip 堆叠筹码
         */
        function getCostByChip(chip) {
            var result = 0,
                sum = 0;
            for (var i = 0; i < factor; i++) {
                var x = xdata[i].toPrecision(12) / 1;
                if (sum + x > chip) {
                    result = minprice + i * accuracy;
                    break;
                }
                sum += x;
            }
            return result;
        }

        /**
         * 筹码分布数据
         */
        function CYQData() {
            /**
             * 筹码堆叠
             * @type {Array.<number>}
             */
            this.x = arguments[0];
            /**
             * 价格分布
             * @type {Array.<number>}
             */
            this.y = arguments[1];
            /**
             * 获利比例
             * @type {number}
             */
            this.benefitPart = arguments[2];
            /**
             * 平均成本
             * @type {number}
             */
            this.avgCost = arguments[3];
            /**
             * 百分比筹码
             * @type {{Object.<string, {{priceRange: number[], concentration: number}}>}}
             */
            this.percentChips = arguments[4];
            /**
             * 计算指定百分比的筹码
             * @param {number} percent 百分比大于0，小于1
             */
            this.computePercentChips = function (percent) {
                if (percent > 1 || percent < 0) throw 'argument "percent" out of range';
                var ps = [(1 - percent) / 2, (1 + percent) / 2];
                var pr = [getCostByChip(totalChips * ps[0]), getCostByChip(totalChips * ps[1])];
                return {
                    priceRange: [pr[0].toFixed(2), pr[1].toFixed(2)],
                    concentration: pr[0] + pr[1] === 0 ? 0 : (pr[1] - pr[0]) / (pr[0] + pr[1])
                };
            };
            /**
             * 获取指定价格的获利比例
             * @param {number} price 价格
             */
            this.getBenefitPart = function (price) {
                var below = 0;
                for (var i = 0; i < factor; i++) {
                    var x = xdata[i].toPrecision(12) / 1;
                    if (price >= minprice + i * accuracy) {
                        below += x;
                    }
                }
                return totalChips == 0 ? 0 : below / totalChips;
            };
        }
    }


    function createNumberArray(count) {
        var array = [];
        for (var i = 0; i < count; i++) {
            array.push(0);
        }
        return array;
    }
    """
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(html_str)
    adjust_dict = {"qfq": "1", "hfq": "2", "": "0"}
    market_code = 1 if symbol.startswith("6") else 0
    url = "https://push2his.eastmoney.com/api/qt/stock/kline/get"
    params = {
        "secid": f"{market_code}.{symbol}",
        "fields1": "f1,f2,f3,f4,f5,f6",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
        "klt": "101",
        "fqt": adjust_dict[adjust],
        "end": datetime.now().date().strftime("%Y%m%d"),
        "lmt": "210",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]["klines"]])
    temp_df.columns = [
        "date",
        "open",
        "close",
        "high",
        "low",
        "volume",
        "volume_money",
        "zf",
        "zdf",
        "zde",
        "hsl",
    ]
    for item in temp_df.columns[1:]:
        temp_df[item] = pd.to_numeric(temp_df[item])
    temp_df["index"] = range(0, len(temp_df))
    records = temp_df.to_dict(orient="records")
    date_list = []
    benefit_part = []
    avg_cost = []
    pct_70_low = []
    pct_70_high = []
    pct_90_low = []
    pct_90_high = []
    pct_70_con = []
    pct_90_con = []
    for i in range(0, len(records)):
        mcode = js_code.call("CYQCalculator", i, records)
        date_list.append(records[i]["date"])
        benefit_part.append(mcode["benefitPart"])
        avg_cost.append(mcode["avgCost"])
        pct_70_low.append(mcode["percentChips"]["70"]["priceRange"][0])
        pct_70_high.append(mcode["percentChips"]["70"]["priceRange"][1])
        pct_90_low.append(mcode["percentChips"]["90"]["priceRange"][0])
        pct_90_high.append(mcode["percentChips"]["90"]["priceRange"][1])
        pct_70_con.append(mcode["percentChips"]["70"]["concentration"])
        pct_90_con.append(mcode["percentChips"]["90"]["concentration"])
    temp_df = pd.DataFrame(
        [
            date_list,
            benefit_part,
            avg_cost,
            pct_90_low,
            pct_90_high,
            pct_90_con,
            pct_70_low,
            pct_70_high,
            pct_70_con,
        ]
    ).T
    temp_df.columns = [
        "日期",
        "获利比例",
        "平均成本",
        "90成本-低",
        "90成本-高",
        "90集中度",
        "70成本-低",
        "70成本-高",
        "70集中度",
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["获利比例"] = pd.to_numeric(temp_df["获利比例"], errors="coerce")
    temp_df["平均成本"] = pd.to_numeric(temp_df["平均成本"], errors="coerce")
    temp_df["90成本-低"] = pd.to_numeric(temp_df["90成本-低"], errors="coerce")
    temp_df["90成本-高"] = pd.to_numeric(temp_df["90成本-高"], errors="coerce")
    temp_df["90集中度"] = pd.to_numeric(temp_df["90集中度"], errors="coerce")
    temp_df["70成本-低"] = pd.to_numeric(temp_df["70成本-低"], errors="coerce")
    temp_df["70成本-高"] = pd.to_numeric(temp_df["70成本-高"], errors="coerce")
    temp_df["70集中度"] = pd.to_numeric(temp_df["70集中度"], errors="coerce")
    temp_df = temp_df.iloc[-90:, :].copy()
    temp_df.reset_index(inplace=True, drop=True)
    return temp_df


if __name__ == "__main__":
    stock_cyq_em_df = stock_cyq_em(symbol="000001", adjust="")
    print(stock_cyq_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/17 19:00
Desc: 数库-A股新闻情绪指数
https://www.chinascope.com/reasearch.html
"""

import pandas as pd
import requests


def index_news_sentiment_scope() -> pd.DataFrame:
    """
    数库-A股新闻情绪指数
    https://www.chinascope.com/reasearch.html
    :return: A股新闻情绪指数
    :rtype: pandas.DataFrame
    """
    url = "https://www.chinascope.com/inews/senti/index"
    params = {"period": "YEAR"}
    r = requests.get(url=url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json)
    temp_df.rename(
        columns={
            "tradeDate": "日期",
            "maIndex1": "市场情绪指数",
            "marketClose": "沪深300指数",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "日期",
            "市场情绪指数",
            "沪深300指数",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["市场情绪指数"] = pd.to_numeric(temp_df["市场情绪指数"], errors="coerce")
    temp_df["沪深300指数"] = pd.to_numeric(temp_df["沪深300指数"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    index_news_sentiment_scope_df = index_news_sentiment_scope()
    print(index_news_sentiment_scope_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/11/20 120:15
Desc: 个股新闻数据
https://so.eastmoney.com/news/s?keyword=603777
"""

import json

import pandas as pd
import requests


def stock_news_em(symbol: str = "603777") -> pd.DataFrame:
    """
    东方财富-个股新闻-最近 100 条新闻
    https://so.eastmoney.com/news/s?keyword=603777
    :param symbol: 股票代码
    :type symbol: str
    :return: 个股新闻
    :rtype: pandas.DataFrame
    """
    url = "https://search-api-web.eastmoney.com/search/jsonp"
    inner_param = {
        "uid": "",
        "keyword": symbol,
        "type": ["cmsArticleWebOld"],
        "client": "web",
        "clientType": "web",
        "clientVersion": "curr",
        "param": {
            "cmsArticleWebOld": {
                "searchScope": "default",
                "sort": "default",
                "pageIndex": 1,
                "pageSize": 10,
                "preTag": "<em>",
                "postTag": "</em>"
            }
        }
    }
    params = {
        "cb": "jQuery35101792940631092459_1764599530165",
        "param": json.dumps(inner_param, ensure_ascii=False),  # 保留中文,
        "_": "1764599530176"
    }
    headers = {
        "accept": "*/*",
        "accept-encoding": "gzip, deflate, br, zstd",
        "accept-language": "en,zh-CN;q=0.9,zh;q=0.8",
        "cache-control": "no-cache",
        "connection": "keep-alive",
        "cookie": "qgqp_b_id=652bf4c98a74e210088f372a17d4e27b; st_nvi=ulN5JAj9FUocz3p4klMME9f20; emshistory=%5B%22603777%22%5D; nid18=010d039dd427dc4d187090491f47d7ad; nid18_create_time=1764582801999; gviem=gSdeY51VWSuTzM3kWaagtf560; gviem_create_time=1764582801999; st_si=55269775884615; st_pvi=66803244437563; st_sp=2025-11-19%2014%3A19%3A16; st_inirUrl=https%3A%2F%2Fso.eastmoney.com%2Fnews%2Fs; st_sn=2; st_psi=20251201223210488-118000300905-0940816858; st_asi=delete",
        "host": "search-api-web.eastmoney.com",
        "pragma": "no-cache",
        "referer": "https://so.eastmoney.com/news/s?keyword=603777",
        "sec-ch-ua": "\"Chromium\";v=\"142\", \"Google Chrome\";v=\"142\", \"Not_A Brand\";v=\"99\"",
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": "\"Windows\"",
        "sec-fetch-dest": "script",
        "sec-fetch-mode": "no-cors",
        "sec-fetch-site": "same-site",
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/142.0.0.0 Safari/537.36"
    }
    r = requests.get(url, params=params, headers=headers)
    data_text = r.text
    data_json = json.loads(
        data_text.strip("jQuery35101792940631092459_1764599530165(")[:-1]
    )
    temp_df = pd.DataFrame(data_json["result"]["cmsArticleWebOld"])
    temp_df["url"] = "http://finance.eastmoney.com/a/" + temp_df["code"] + ".html"
    temp_df.rename(
        columns={
            "date": "发布时间",
            "mediaName": "文章来源",
            "code": "-",
            "title": "新闻标题",
            "content": "新闻内容",
            "url": "新闻链接",
            "image": "-",
        },
        inplace=True,
    )
    temp_df["关键词"] = symbol
    temp_df = temp_df[
        [
            "关键词",
            "新闻标题",
            "新闻内容",
            "发布时间",
            "文章来源",
            "新闻链接",
        ]
    ]
    temp_df["新闻标题"] = (
        temp_df["新闻标题"]
        .str.replace(r"\(<em>", "", regex=True)
        .str.replace(r"</em>\)", "", regex=True)
    )
    temp_df["新闻标题"] = (
        temp_df["新闻标题"]
        .str.replace(r"<em>", "", regex=True)
        .str.replace(r"</em>", "", regex=True)
    )
    temp_df["新闻内容"] = (
        temp_df["新闻内容"]
        .str.replace(r"\(<em>", "", regex=True)
        .str.replace(r"</em>\)", "", regex=True)
    )
    temp_df["新闻内容"] = (
        temp_df["新闻内容"]
        .str.replace(r"<em>", "", regex=True)
        .str.replace(r"</em>", "", regex=True)
    )
    temp_df["新闻内容"] = temp_df["新闻内容"].str.replace(r"\u3000", "", regex=True)
    temp_df["新闻内容"] = temp_df["新闻内容"].str.replace(r"\r\n", " ", regex=True)
    return temp_df


if __name__ == "__main__":
    stock_news_em_df = stock_news_em(symbol="603777")
    print(stock_news_em_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2025/3/19 15:30
Desc: 新股和风险警示股
新浪-行情中心-沪深股市-次新股
https://vip.stock.finance.sina.com.cn/mkt/#new_stock
东方财富网-行情中心-沪深个股-风险警示板
https://quote.eastmoney.com/center/gridlist.html#st_board
"""

import math

import pandas as pd
import requests

from akshare.utils.func import fetch_paginated_data


def stock_zh_a_st_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-沪深个股-风险警示板
    https://quote.eastmoney.com/center/gridlist.html#st_board
    :return: 风险警示板
    :rtype: pandas.DataFrame
    """
    url = "https://40.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f3",
        "fs": "m:0 f:4,m:1 f:4",
        "fields": "f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,f20,"
        "f21,f23,f24,f25,f22,f11,f62,f128,f136,f115,f152",
    }
    temp_df = fetch_paginated_data(url, params)
    temp_df.columns = [
        "序号",
        "最新价",
        "涨跌幅",
        "涨跌额",
        "成交量",
        "成交额",
        "振幅",
        "换手率",
        "市盈率-动态",
        "量比",
        "_",
        "代码",
        "_",
        "名称",
        "最高",
        "最低",
        "今开",
        "昨收",
        "_",
        "_",
        "_",
        "市净率",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "涨跌额",
            "成交量",
            "成交额",
            "振幅",
            "最高",
            "最低",
            "今开",
            "昨收",
            "量比",
            "换手率",
            "市盈率-动态",
            "市净率",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce")
    temp_df["量比"] = pd.to_numeric(temp_df["量比"], errors="coerce")
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
    return temp_df


def stock_zh_a_new_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-沪深个股-新股
    https://quote.eastmoney.com/center/gridlist.html#newshares
    :return: 新股
    :rtype: pandas.DataFrame
    """
    url = "https://40.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f26",
        "fs": "m:0 f:8,m:1 f:8",
        "fields": "f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,"
        "f20,f21,f23,f24,f25,f22,f11,f62,f128,f136,f115,f152",
    }
    temp_df = fetch_paginated_data(url, params)
    temp_df.columns = [
        "序号",
        "最新价",
        "涨跌幅",
        "涨跌额",
        "成交量",
        "成交额",
        "振幅",
        "换手率",
        "市盈率-动态",
        "量比",
        "_",
        "代码",
        "_",
        "名称",
        "最高",
        "最低",
        "今开",
        "昨收",
        "_",
        "_",
        "_",
        "市净率",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "涨跌额",
            "成交量",
            "成交额",
            "振幅",
            "最高",
            "最低",
            "今开",
            "昨收",
            "量比",
            "换手率",
            "市盈率-动态",
            "市净率",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce")
    temp_df["量比"] = pd.to_numeric(temp_df["量比"], errors="coerce")
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
    return temp_df


def stock_zh_a_stop_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-沪深个股-两网及退市
    https://quote.eastmoney.com/center/gridlist.html#staq_net_board
    :return: 两网及退市
    :rtype: pandas.DataFrame
    """
    url = "https://40.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f3",
        "fs": "m:0 s:3",
        "fields": "f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,"
        "f18,f20,f21,f23,f24,f25,f22,f11,f62,f128,f136,f115,f152",
    }
    temp_df = fetch_paginated_data(url, params)
    temp_df.columns = [
        "序号",
        "最新价",
        "涨跌幅",
        "涨跌额",
        "成交量",
        "成交额",
        "振幅",
        "换手率",
        "市盈率-动态",
        "量比",
        "_",
        "代码",
        "_",
        "名称",
        "最高",
        "最低",
        "今开",
        "昨收",
        "_",
        "_",
        "_",
        "市净率",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "涨跌额",
            "成交量",
            "成交额",
            "振幅",
            "最高",
            "最低",
            "今开",
            "昨收",
            "量比",
            "换手率",
            "市盈率-动态",
            "市净率",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce")
    temp_df["量比"] = pd.to_numeric(temp_df["量比"], errors="coerce")
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
    return temp_df


def stock_zh_a_new() -> pd.DataFrame:
    """
    新浪财经-行情中心-沪深股市-次新股
    https://vip.stock.finance.sina.com.cn/mkt/#new_stock
    :return: 次新股行情数据
    :rtype: pandas.DataFrame
    """
    url = "https://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeStockCount"
    params = {"node": "new_stock"}
    r = requests.get(url, params=params)
    total_page = math.ceil(int(r.json()) / 80)
    url = "https://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeData"
    big_df = pd.DataFrame()
    for page in range(1, total_page + 1):
        params = {
            "page": str(page),
            "num": "80",
            "sort": "symbol",
            "asc": "1",
            "node": "new_stock",
            "symbol": "",
            "_s_r_a": "page",
        }
        r = requests.get(url, params=params)
        r.encoding = "gb2312"
        data_json = r.json()
        temp_df = pd.DataFrame(data_json)
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df = big_df[
        [
            "symbol",
            "code",
            "name",
            "open",
            "high",
            "low",
            "volume",
            "amount",
            "mktcap",
            "turnoverratio",
        ]
    ]
    big_df["open"] = pd.to_numeric(big_df["open"])
    big_df["high"] = pd.to_numeric(big_df["high"])
    big_df["low"] = pd.to_numeric(big_df["low"])
    return big_df


if __name__ == "__main__":
    stock_zh_a_st_em_df = stock_zh_a_st_em()
    print(stock_zh_a_st_em_df)

    stock_zh_a_new_em_df = stock_zh_a_new_em()
    print(stock_zh_a_new_em_df)

    stock_zh_a_stop_em_df = stock_zh_a_stop_em()
    print(stock_zh_a_stop_em_df)

    stock_zh_a_new_df = stock_zh_a_new()
    print(stock_zh_a_new_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/8/28 15:20
Desc: 世界各大城市生活成本数据
https://expatistan.com/cost-of-living/index
"""

from io import StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup


def _get_region() -> dict:
    """
    获取主要板块, 一般不调用
    :return: 主要板块
    :rtype: dict
    """
    url = "https://www.expatistan.com/cost-of-living/index"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    half_url_list = [
        item["href"]
        for item in soup.find(name="ul", attrs={"class": "regions"}).find_all("a")
    ]
    name_list = [
        item["href"].split("/")[-1]
        for item in soup.find(name="ul", attrs={"class": "regions"}).find_all("a")
    ]
    name_url_dict = dict(zip(name_list, half_url_list))
    name_url_dict["world"] = "/cost-of-living/index"
    return name_url_dict


def cost_living(symbol: str = "world") -> pd.DataFrame:
    """
    国家或地区生活成本数据
    https://expatistan.com/cost-of-living/index
    :param symbol: choice of {"europe", "north-america", "latin-america",
    "asia", "middle-east", "africa", "oceania", "world"}
    :type symbol: str
    :return: 国家或地区生活成本数据
    :rtype: pandas.DataFrame
    """
    name_url_map = {
        "europe": "/cost-of-living/index/europe",
        "north-america": "/cost-of-living/index/north-america",
        "latin-america": "/cost-of-living/index/latin-america",
        "asia": "/cost-of-living/index/asia",
        "middle-east": "/cost-of-living/index/middle-east",
        "africa": "/cost-of-living/index/africa",
        "oceania": "/cost-of-living/index/oceania",
        "world": "/cost-of-living/index",
    }
    url = f"https://www.expatistan.com{name_url_map[symbol]}"
    r = requests.get(url)
    temp_df = pd.read_html(StringIO(r.text))[0]
    temp_df.columns = ["rank", "city", "index"]
    return temp_df


if __name__ == "__main__":
    cost_living_df = cost_living(symbol="world")
    print(cost_living_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/12/27 15:47
Desc: 中国公路物流运价、运量指数
http://index.0256.cn/expx.htm
"""

import pandas as pd
import requests


def index_price_cflp(symbol: str = "周指数") -> pd.DataFrame:
    """
    中国公路物流运价指数
    http://index.0256.cn/expx.htm
    :param symbol: choice of {"周指数", "月指数", "季度指数", "年度指数"}
    :type symbol: str
    :return: 中国公路物流运价指数
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "周指数": "2",
        "月指数": "3",
        "季度指数": "4",
        "年度指数": "5",
    }
    url = "http://index.0256.cn/expcenter_trend.action"
    params = {
        "marketId": "1",
        "attribute1": "5",
        "exponentTypeId": symbol_map[symbol],
        "cateId": "2",
        "attribute2": "华北",
        "city": "",
        "startLine": "",
        "endLine": "",
    }
    headers = {
        "Origin": "http://index.0256.cn",
        "Referer": "http://index.0256.cn/expx.htm",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/90.0.4430.212 Safari/537.36",
    }
    r = requests.post(url, data=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(
        [
            data_json["chart1"]["xLebal"],
            data_json["chart1"]["yLebal"],
            data_json["chart2"]["yLebal"],
            data_json["chart3"]["yLebal"],
        ]
    ).T
    temp_df.columns = ["日期", "定基指数", "环比指数", "同比指数"]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["定基指数"] = pd.to_numeric(temp_df["定基指数"], errors="coerce")
    temp_df["环比指数"] = pd.to_numeric(temp_df["环比指数"], errors="coerce")
    temp_df["同比指数"] = pd.to_numeric(temp_df["同比指数"], errors="coerce")
    return temp_df


def index_volume_cflp(symbol: str = "月指数") -> pd.DataFrame:
    """
    中国公路物流运量指数
    http://index.0256.cn/expx.htm
    :param symbol: choice of {"月指数", "季度指数", "年度指数"}
    :type symbol: str
    :return: 中国公路物流运量指数
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "月指数": "3",
        "季度指数": "4",
        "年度指数": "5",
    }
    url = "http://index.0256.cn/volume_query.action"
    params = {
        "type": "1",
        "marketId": "1",
        "expTypeId": symbol_map[symbol],
        "startDate1": "",
        "endDate1": "",
        "city": "",
        "startDate3": "",
        "endDate3": "",
    }
    headers = {
        "Origin": "http://index.0256.cn",
        "Referer": "http://index.0256.cn/expx.htm",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/90.0.4430.212 Safari/537.36",
    }
    r = requests.post(url, data=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(
        [
            data_json["chart1"]["xLebal"],
            data_json["chart1"]["yLebal"],
            data_json["chart2"]["yLebal"],
            data_json["chart3"]["yLebal"],
        ]
    ).T
    temp_df.columns = ["日期", "定基指数", "环比指数", "同比指数"]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["定基指数"] = pd.to_numeric(temp_df["定基指数"], errors="coerce")
    temp_df["环比指数"] = pd.to_numeric(temp_df["环比指数"], errors="coerce")
    temp_df["同比指数"] = pd.to_numeric(temp_df["同比指数"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    index_price_cflp_df = index_price_cflp(symbol="周指数")
    print(index_price_cflp_df)

    index_price_cflp_df = index_price_cflp(symbol="月指数")
    print(index_price_cflp_df)

    index_price_cflp_df = index_price_cflp(symbol="季度指数")
    print(index_price_cflp_df)

    index_price_cflp_df = index_price_cflp(symbol="年度指数")
    print(index_price_cflp_df)

    index_volume_cflp_df = index_volume_cflp(symbol="月指数")
    print(index_volume_cflp_df)

    index_volume_cflp_df = index_volume_cflp(symbol="季度指数")
    print(index_volume_cflp_df)

    index_volume_cflp_df = index_volume_cflp(symbol="年度指数")
    print(index_volume_cflp_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/8/3 20:00
Desc: 乘联会
http://data.cpcadata.com/FuelMarket
"""

import pandas as pd
import requests


def car_market_total_cpca(
    symbol: str = "狭义乘用车", indicator: str = "产量"
) -> pd.DataFrame:
    """
    乘联会-统计数据-总体市场
    http://data.cpcadata.com/TotalMarket
    :param symbol: choice of {"狭义乘用车", "广义乘用车"}
    :type symbol: str
    :param indicator: choice of {"产量", "批发", "零售", "出口"}
    :type indicator: str
    :return: 统计数据-总体市场
    :rtype: pandas.DataFrame
    """
    url = "http://data.cpcadata.com/api/chartlist"
    params = {"charttype": "1"}
    r = requests.get(url, params=params)
    data_json = r.json()
    big_df = pd.DataFrame()
    if symbol == "狭义乘用车":
        temp_df = pd.DataFrame(data_json[0]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[0]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[2]])
            try:
                temp_current_year_list.append(item[temp_df.columns[1]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        if indicator == "产量":
            big_df = pd.DataFrame(
                [temp_current_year_df.iloc[:, 0], temp_previous_year_df.iloc[:, 0]]
            ).T
            big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
            big_df["月份"] = temp_df["month"]
            big_df = big_df[
                [
                    "月份",
                    temp_df.columns[2],
                    temp_df.columns[1],
                ]
            ]
        elif indicator == "批发":
            big_df = pd.DataFrame(
                [temp_current_year_df.iloc[:, 1], temp_previous_year_df.iloc[:, 1]]
            ).T
            big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
            big_df["月份"] = temp_df["month"]
            big_df = big_df[
                [
                    "月份",
                    temp_df.columns[2],
                    temp_df.columns[1],
                ]
            ]
        elif indicator == "零售":
            big_df = pd.DataFrame(
                [temp_current_year_df.iloc[:, 2], temp_previous_year_df.iloc[:, 2]]
            ).T
            big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
            big_df["月份"] = temp_df["month"]
            big_df = big_df[
                [
                    "月份",
                    temp_df.columns[2],
                    temp_df.columns[1],
                ]
            ]
        elif indicator == "出口":
            big_df = pd.DataFrame(
                [temp_current_year_df.iloc[:, 3], temp_previous_year_df.iloc[:, 3]]
            ).T
            big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
            big_df["月份"] = temp_df["month"]
            big_df = big_df[
                [
                    "月份",
                    temp_df.columns[2],
                    temp_df.columns[1],
                ]
            ]
    else:
        temp_df = pd.DataFrame(data_json[1]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[1]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[2]])
            try:
                temp_current_year_list.append(item[temp_df.columns[1]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        if indicator == "产量":
            big_df = pd.DataFrame(
                [temp_current_year_df.iloc[:, 0], temp_previous_year_df.iloc[:, 0]]
            ).T
            big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
            big_df["月份"] = temp_df["month"]
            big_df = big_df[
                [
                    "月份",
                    temp_df.columns[2],
                    temp_df.columns[1],
                ]
            ]
        elif indicator == "批发":
            big_df = pd.DataFrame(
                [temp_current_year_df.iloc[:, 1], temp_previous_year_df.iloc[:, 1]]
            ).T
            big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
            big_df["月份"] = temp_df["month"]
            big_df = big_df[
                [
                    "月份",
                    temp_df.columns[2],
                    temp_df.columns[1],
                ]
            ]
        elif indicator == "零售":
            big_df = pd.DataFrame(
                [temp_current_year_df.iloc[:, 2], temp_previous_year_df.iloc[:, 2]]
            ).T
            big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
            big_df["月份"] = temp_df["month"]
            big_df = big_df[
                [
                    "月份",
                    temp_df.columns[2],
                    temp_df.columns[1],
                ]
            ]
        elif indicator == "出口":
            big_df = pd.DataFrame(
                [temp_current_year_df.iloc[:, 3], temp_previous_year_df.iloc[:, 3]]
            ).T
            big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
            big_df["月份"] = temp_df["month"]
            big_df = big_df[
                [
                    "月份",
                    temp_df.columns[2],
                    temp_df.columns[1],
                ]
            ]

    return big_df


def __car_market_man_rank_cpca_pifa(symbol: str = "狭义乘用车-累计") -> pd.DataFrame:
    """
    乘联会-统计数据-厂商排名
    http://data.cpcadata.com/ManRank
    :param symbol: choice of {"狭义乘用车-单月", "狭义乘用车-累计", "广义乘用车-单月", "广义乘用车-累计"}
    :type symbol: str
    :return: 统计数据-厂商排名
    :rtype: pandas.DataFrame
    """
    url = "http://data.cpcadata.com/api/chartlist"
    params = {"charttype": "2"}
    r = requests.get(url, params=params)
    data_json = r.json()
    big_df = pd.DataFrame()
    if symbol == "狭义乘用车-累计":
        temp_df = pd.DataFrame(data_json[0]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[0]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[2]])
            try:
                temp_current_year_list.append(item[temp_df.columns[1]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 0], temp_previous_year_df.iloc[:, 0]]
        ).T
        big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
        big_df["厂商"] = temp_df["厂商"]
        big_df = big_df[
            [
                "厂商",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    elif symbol == "狭义乘用车-单月":
        temp_df = pd.DataFrame(data_json[1]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[1]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[2]])
            try:
                temp_current_year_list.append(item[temp_df.columns[1]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 0], temp_previous_year_df.iloc[:, 0]]
        ).T
        big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
        big_df["厂商"] = temp_df["厂商"]
        big_df = big_df[
            [
                "厂商",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    elif symbol == "广义乘用车-累计":
        temp_df = pd.DataFrame(data_json[2]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[2]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[2]])
            try:
                temp_current_year_list.append(item[temp_df.columns[1]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 0], temp_previous_year_df.iloc[:, 0]]
        ).T
        big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
        big_df["厂商"] = temp_df["厂商"]
        big_df = big_df[
            [
                "厂商",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    elif symbol == "广义乘用车-单月":
        temp_df = pd.DataFrame(data_json[3]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[3]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[2]])
            try:
                temp_current_year_list.append(item[temp_df.columns[1]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 0], temp_previous_year_df.iloc[:, 0]]
        ).T
        big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
        big_df["厂商"] = temp_df["厂商"]
        big_df = big_df[
            [
                "厂商",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    return big_df


def __car_market_man_rank_cpca_lingshou(
    symbol: str = "狭义乘用车-累计",
) -> pd.DataFrame:
    """
    乘联会-统计数据-厂商排名
    http://data.cpcadata.com/ManRank
    :param symbol: choice of {"狭义乘用车-单月", "狭义乘用车-累计", "广义乘用车-单月", "广义乘用车-累计"}
    :type symbol: str
    :return: 统计数据-厂商排名
    :rtype: pandas.DataFrame
    """
    url = "http://data.cpcadata.com/api/chartlist_2"
    params = {"charttype": "2"}
    r = requests.get(url, params=params)
    data_json = r.json()
    big_df = pd.DataFrame()
    if symbol == "狭义乘用车-累计":
        temp_df = pd.DataFrame(data_json[0]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[0]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[2]])
            try:
                temp_current_year_list.append(item[temp_df.columns[1]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 1], temp_previous_year_df.iloc[:, 1]]
        ).T
        big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
        big_df["厂商"] = temp_df["厂商"]
        big_df = big_df[
            [
                "厂商",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    elif symbol == "狭义乘用车-单月":
        temp_df = pd.DataFrame(data_json[1]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[1]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[2]])
            try:
                temp_current_year_list.append(item[temp_df.columns[1]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 1], temp_previous_year_df.iloc[:, 1]]
        ).T
        big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
        big_df["厂商"] = temp_df["厂商"]
        big_df = big_df[
            [
                "厂商",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    elif symbol == "广义乘用车-累计":
        temp_df = pd.DataFrame(data_json[2]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[2]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[2]])
            try:
                temp_current_year_list.append(item[temp_df.columns[1]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 1], temp_previous_year_df.iloc[:, 1]]
        ).T
        big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
        big_df["厂商"] = temp_df["厂商"]
        big_df = big_df[
            [
                "厂商",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    elif symbol == "广义乘用车-单月":
        temp_df = pd.DataFrame(data_json[3]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[3]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[2]])
            try:
                temp_current_year_list.append(item[temp_df.columns[1]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 1], temp_previous_year_df.iloc[:, 1]]
        ).T
        big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
        big_df["厂商"] = temp_df["厂商"]
        big_df = big_df[
            [
                "厂商",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    return big_df


def car_market_man_rank_cpca(
    symbol: str = "狭义乘用车-单月", indicator: str = "批发"
) -> pd.DataFrame:
    """
    乘联会-统计数据-厂商排名
    http://data.cpcadata.com/ManRank
    :param symbol: choice of {"狭义乘用车-单月", "狭义乘用车-累计", "广义乘用车-单月", "广义乘用车-累计"}
    :type symbol: str
    :param indicator: choice of {"批发", "零售"}
    :type indicator: str
    :return: 统计数据-厂商排名
    :rtype: pandas.DataFrame
    """
    if indicator == "批发":
        temp_df = __car_market_man_rank_cpca_pifa(symbol=symbol)
        return temp_df
    else:
        temp_df = __car_market_man_rank_cpca_lingshou(symbol=symbol)
        return temp_df


def __car_market_cate_cpca_pifa(symbol: str = "MPV") -> pd.DataFrame:
    """
    乘联会-统计数据-车型大类
    http://data.cpcadata.com/CategoryMarket
    :param symbol: choice of {"轿车", "MPV", "SUV", "占比"}
    :type symbol: str
    :return: 统计数据-车型大类
    :rtype: pandas.DataFrame
    """
    url = "http://data.cpcadata.com/api/chartlist"
    params = {"charttype": "3"}
    r = requests.get(url, params=params)
    data_json = r.json()
    big_df = pd.DataFrame()
    if symbol == "MPV":
        temp_df = pd.DataFrame(data_json[0]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[0]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[1]])
            try:
                temp_current_year_list.append(item[temp_df.columns[2]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 1], temp_previous_year_df.iloc[:, 1]]
        ).T
        big_df.columns = [temp_df.columns[2], temp_df.columns[1]]
        big_df["月份"] = temp_df["month"]
        big_df = big_df[
            [
                "月份",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    elif symbol == "SUV":
        temp_df = pd.DataFrame(data_json[1]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[1]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[1]])
            try:
                temp_current_year_list.append(item[temp_df.columns[2]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 1], temp_previous_year_df.iloc[:, 1]]
        ).T
        big_df.columns = [temp_df.columns[2], temp_df.columns[1]]
        big_df["月份"] = temp_df["month"]
        big_df = big_df[
            [
                "月份",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    elif symbol == "轿车":
        temp_df = pd.DataFrame(data_json[2]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[2]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[1]])
            try:
                temp_current_year_list.append(item[temp_df.columns[2]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 1], temp_previous_year_df.iloc[:, 1]]
        ).T
        big_df.columns = [temp_df.columns[2], temp_df.columns[1]]
        big_df["月份"] = temp_df["month"]
        big_df = big_df[
            [
                "月份",
                temp_df.columns[1],
                temp_df.columns[2],
            ]
        ]
    elif symbol == "占比":
        temp_df = pd.DataFrame(data_json[3]["dataList"])
        temp_mpv_year_list = []
        temp_suv_year_list = []
        temp_jiaoche_year_list = []
        for item in data_json[3]["dataList"]:
            temp_mpv_year_list.append(item[temp_df.columns[1]])
            try:
                temp_suv_year_list.append(item[temp_df.columns[2]])
                temp_jiaoche_year_list.append(item[temp_df.columns[3]])
            except:  # noqa: E722
                continue
        temp_mpv_year_df = pd.DataFrame(temp_mpv_year_list)
        temp_suv_year_df = pd.DataFrame(temp_suv_year_list)
        temp_jiaoche_year_df = pd.DataFrame(temp_jiaoche_year_list)
        big_df = pd.DataFrame(
            [
                temp_mpv_year_df.iloc[:, 2],
                temp_suv_year_df.iloc[:, 2],
                temp_jiaoche_year_df.iloc[:, 2],
            ]
        ).T
        big_df.columns = [temp_df.columns[1], temp_df.columns[2], temp_df.columns[3]]
        big_df["月份"] = temp_df["月份"]
        big_df = big_df[
            ["月份", temp_df.columns[1], temp_df.columns[2], temp_df.columns[3]]
        ]
    return big_df


def __car_market_cate_cpca_lingshou(
    symbol: str = "狭义乘用车-累计",
) -> pd.DataFrame:
    """
    乘联会-统计数据-车型大类
    http://data.cpcadata.com/CategoryMarket
    :param symbol: choice of {"轿车", "MPV", "SUV", "占比"}
    :type symbol: str
    :return: 统计数据-车型大类
    :rtype: pandas.DataFrame
    """
    url = "http://data.cpcadata.com/api/chartlist"
    params = {"charttype": "3"}
    r = requests.get(url, params=params)
    data_json = r.json()
    big_df = pd.DataFrame()
    if symbol == "MPV":
        temp_df = pd.DataFrame(data_json[0]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[0]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[1]])
            try:
                temp_current_year_list.append(item[temp_df.columns[2]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 2], temp_previous_year_df.iloc[:, 2]]
        ).T
        big_df.columns = [temp_df.columns[2], temp_df.columns[1]]
        big_df["月份"] = temp_df["month"]
        big_df = big_df[
            [
                "月份",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    elif symbol == "SUV":
        temp_df = pd.DataFrame(data_json[1]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[1]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[1]])
            try:
                temp_current_year_list.append(item[temp_df.columns[2]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 2], temp_previous_year_df.iloc[:, 2]]
        ).T
        big_df.columns = [temp_df.columns[2], temp_df.columns[1]]
        big_df["月份"] = temp_df["month"]
        big_df = big_df[
            [
                "月份",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    elif symbol == "轿车":
        temp_df = pd.DataFrame(data_json[2]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[2]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[1]])
            try:
                temp_current_year_list.append(item[temp_df.columns[2]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 2], temp_previous_year_df.iloc[:, 2]]
        ).T
        big_df.columns = [temp_df.columns[2], temp_df.columns[1]]
        big_df["月份"] = temp_df["month"]
        big_df = big_df[
            [
                "月份",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    elif symbol == "占比":
        temp_df = pd.DataFrame(data_json[3]["dataList"])
        temp_mpv_year_list = []
        temp_suv_year_list = []
        temp_jiaoche_year_list = []
        for item in data_json[3]["dataList"]:
            temp_mpv_year_list.append(item[temp_df.columns[1]])
            try:
                temp_suv_year_list.append(item[temp_df.columns[2]])
                temp_jiaoche_year_list.append(item[temp_df.columns[3]])
            except:  # noqa: E722
                continue
        temp_mpv_year_df = pd.DataFrame(temp_mpv_year_list)
        temp_suv_year_df = pd.DataFrame(temp_suv_year_list)
        temp_jiaoche_year_df = pd.DataFrame(temp_jiaoche_year_list)
        big_df = pd.DataFrame(
            [
                temp_mpv_year_df.iloc[:, 3],
                temp_suv_year_df.iloc[:, 3],
                temp_jiaoche_year_df.iloc[:, 3],
            ]
        ).T
        big_df.columns = [temp_df.columns[1], temp_df.columns[2], temp_df.columns[3]]
        big_df["月份"] = temp_df["月份"]
        big_df = big_df[
            ["月份", temp_df.columns[1], temp_df.columns[2], temp_df.columns[3]]
        ]
    return big_df


def car_market_cate_cpca(symbol: str = "轿车", indicator: str = "批发") -> pd.DataFrame:
    """
    乘联会-统计数据-车型大类
    http://data.cpcadata.com/CategoryMarket
    :param symbol: choice of {"轿车", "MPV", "SUV", "占比"}
    :type symbol: str
    :param indicator: choice of {"批发", "零售"}
    :type indicator: str
    :return: 统计数据-车型大类
    :rtype: pandas.DataFrame
    """
    if indicator == "批发":
        temp_df = __car_market_cate_cpca_pifa(symbol=symbol)
        return temp_df
    else:
        temp_df = __car_market_cate_cpca_lingshou(symbol=symbol)
        return temp_df


def car_market_country_cpca() -> pd.DataFrame:
    """
    乘联会-统计数据-国别细分市场
    http://data.cpcadata.com/CountryMarket
    :return: 统计数据-车型大类
    :rtype: pandas.DataFrame
    """
    url = "http://data.cpcadata.com/api/chartlist"
    params = {"charttype": "4"}
    r = requests.get(url=url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json[0]["dataList"])
    for item in temp_df.columns[1:]:
        temp_list = []
        for item_list in temp_df[item]:
            temp_list.append(item_list[2])
        temp_df[item] = pd.to_numeric(temp_list, errors="coerce")
    return temp_df


def car_market_segment_cpca(symbol: str = "轿车") -> pd.DataFrame:
    """
    乘联会-统计数据-级别细分市场
    http://data.cpcadata.com/SegmentMarket
    :param symbol: choice of {"轿车", "MPV", "SUV"}
    :type symbol: str
    :return: 统计数据-车型大类
    :rtype: pandas.DataFrame
    """
    url = "http://data.cpcadata.com/api/chartlist"
    params = {"charttype": "5"}
    r = requests.get(url=url, params=params)
    data_json = r.json()
    if symbol == "MPV":
        temp_df = pd.DataFrame(data_json[0]["dataList"])
        for item in temp_df.columns[1:]:
            temp_list = []
            for item_list in temp_df[item]:
                temp_list.append(item_list[2])
            temp_df[item] = pd.to_numeric(temp_list, errors="coerce")
    elif symbol == "SUV":
        temp_df = pd.DataFrame(data_json[1]["dataList"])
        for item in temp_df.columns[1:]:
            temp_list = []
            for item_list in temp_df[item]:
                temp_list.append(item_list[2])
            temp_df[item] = pd.to_numeric(temp_list, errors="coerce")
    else:
        temp_df = pd.DataFrame(data_json[2]["dataList"])
        for item in temp_df.columns[1:]:
            temp_list = []
            for item_list in temp_df[item]:
                temp_list.append(item_list[2])
            temp_df[item] = pd.to_numeric(temp_list, errors="coerce")
    return temp_df


def car_market_fuel_cpca(symbol: str = "整体市场") -> pd.DataFrame:
    """
    乘联会-统计数据-新能源细分市场
    :param symbol: choice of {"整体市场", "销量占比-PHEV-BEV", "销量占比-ICE-NEV"}
    :type symbol: str
    https://data.cpcadata.com/FuelMarket
    :return: 新能源细分市场
    :rtype: pandas.DataFrame
    """
    url = "http://data.cpcadata.com/api/chartlist"
    params = {"charttype": "6"}
    r = requests.get(url, params=params)
    data_json = r.json()
    if symbol == "整体市场":
        temp_df = pd.DataFrame(data_json[0]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[0]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[2]])
            try:
                temp_current_year_list.append(item[temp_df.columns[1]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 2], temp_previous_year_df.iloc[:, 2]]
        ).T
        big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
        big_df["月份"] = temp_df["month"]
        big_df = big_df[
            [
                "月份",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    elif symbol == "销量占比-PHEV-BEV":
        temp_df = pd.DataFrame(data_json[1]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[1]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[2]])
            try:
                temp_current_year_list.append(item[temp_df.columns[1]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 2], temp_previous_year_df.iloc[:, 2]]
        ).T
        big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
        big_df["月份"] = temp_df["月份"]
        big_df = big_df[
            [
                "月份",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    else:
        temp_df = pd.DataFrame(data_json[2]["dataList"])
        temp_current_year_list = []
        temp_previous_year_list = []
        for item in data_json[2]["dataList"]:
            temp_previous_year_list.append(item[temp_df.columns[2]])
            try:
                temp_current_year_list.append(item[temp_df.columns[1]])
            except:  # noqa: E722
                continue
        temp_current_year_df = pd.DataFrame(temp_current_year_list)
        temp_previous_year_df = pd.DataFrame(temp_previous_year_list)
        big_df = pd.DataFrame(
            [temp_current_year_df.iloc[:, 2], temp_previous_year_df.iloc[:, 2]]
        ).T
        big_df.columns = [temp_df.columns[1], temp_df.columns[2]]
        big_df["月份"] = temp_df["月份"]
        big_df = big_df[
            [
                "月份",
                temp_df.columns[2],
                temp_df.columns[1],
            ]
        ]
    return big_df


if __name__ == "__main__":
    car_market_total_cpca_df = car_market_total_cpca(
        symbol="狭义乘用车", indicator="产量"
    )
    print(car_market_total_cpca_df)

    car_market_total_cpca_df = car_market_total_cpca(
        symbol="广义乘用车", indicator="产量"
    )
    print(car_market_total_cpca_df)

    car_market_total_cpca_df = car_market_total_cpca(
        symbol="狭义乘用车", indicator="批发"
    )
    print(car_market_total_cpca_df)

    car_market_total_cpca_df = car_market_total_cpca(
        symbol="广义乘用车", indicator="批发"
    )
    print(car_market_total_cpca_df)

    car_market_total_cpca_df = car_market_total_cpca(
        symbol="狭义乘用车", indicator="零售"
    )
    print(car_market_total_cpca_df)

    car_market_total_cpca_df = car_market_total_cpca(
        symbol="广义乘用车", indicator="零售"
    )
    print(car_market_total_cpca_df)

    car_market_total_cpca_df = car_market_total_cpca(
        symbol="狭义乘用车", indicator="出口"
    )
    print(car_market_total_cpca_df)

    car_market_total_cpca_df = car_market_total_cpca(
        symbol="广义乘用车", indicator="出口"
    )
    print(car_market_total_cpca_df)

    car_market_man_rank_cpca_df = car_market_man_rank_cpca(
        symbol="狭义乘用车-单月", indicator="批发"
    )
    print(car_market_man_rank_cpca_df)

    car_market_man_rank_cpca_df = car_market_man_rank_cpca(
        symbol="狭义乘用车-累计", indicator="批发"
    )
    print(car_market_man_rank_cpca_df)

    car_market_man_rank_cpca_df = car_market_man_rank_cpca(
        symbol="广义乘用车-单月", indicator="批发"
    )
    print(car_market_man_rank_cpca_df)

    car_market_man_rank_cpca_df = car_market_man_rank_cpca(
        symbol="广义乘用车-累计", indicator="批发"
    )
    print(car_market_man_rank_cpca_df)

    car_market_man_rank_cpca_df = car_market_man_rank_cpca(
        symbol="狭义乘用车-单月", indicator="零售"
    )
    print(car_market_man_rank_cpca_df)

    car_market_man_rank_cpca_df = car_market_man_rank_cpca(
        symbol="狭义乘用车-累计", indicator="零售"
    )
    print(car_market_man_rank_cpca_df)

    car_market_man_rank_cpca_df = car_market_man_rank_cpca(
        symbol="广义乘用车-单月", indicator="零售"
    )
    print(car_market_man_rank_cpca_df)

    car_market_man_rank_cpca_df = car_market_man_rank_cpca(
        symbol="广义乘用车-累计", indicator="零售"
    )
    print(car_market_man_rank_cpca_df)

    car_market_cate_cpca_df = car_market_cate_cpca(symbol="轿车", indicator="批发")
    print(car_market_cate_cpca_df)

    car_market_cate_cpca_df = car_market_cate_cpca(symbol="MPV", indicator="批发")
    print(car_market_cate_cpca_df)

    car_market_cate_cpca_df = car_market_cate_cpca(symbol="SUV", indicator="批发")
    print(car_market_cate_cpca_df)

    car_market_cate_cpca_df = car_market_cate_cpca(symbol="占比", indicator="批发")
    print(car_market_cate_cpca_df)

    car_market_cate_cpca_df = car_market_cate_cpca(symbol="轿车", indicator="零售")
    print(car_market_cate_cpca_df)

    car_market_cate_cpca_df = car_market_cate_cpca(symbol="MPV", indicator="零售")
    print(car_market_cate_cpca_df)

    car_market_cate_cpca_df = car_market_cate_cpca(symbol="SUV", indicator="零售")
    print(car_market_cate_cpca_df)

    car_market_cate_cpca_df = car_market_cate_cpca(symbol="占比", indicator="零售")
    print(car_market_cate_cpca_df)

    car_market_country_cpca_df = car_market_country_cpca()
    print(car_market_country_cpca_df)

    car_market_segment_cpca_df = car_market_segment_cpca(symbol="轿车")
    print(car_market_segment_cpca_df)

    car_market_segment_cpca_df = car_market_segment_cpca(symbol="MPV")
    print(car_market_segment_cpca_df)

    car_market_segment_cpca_df = car_market_segment_cpca(symbol="SUV")
    print(car_market_segment_cpca_df)

    car_market_fuel_cpca_df = car_market_fuel_cpca(symbol="整体市场")
    print(car_market_fuel_cpca_df)

    car_market_fuel_cpca_df = car_market_fuel_cpca(symbol="销量占比-PHEV-BEV")
    print(car_market_fuel_cpca_df)

    car_market_fuel_cpca_df = car_market_fuel_cpca(symbol="销量占比-ICE-NEV")
    print(car_market_fuel_cpca_df)
